@@ -30,6 +30,8 @@ from pytgpt.utils import default_path
 from pytgpt.utils import AwesomePrompts
 from pytgpt.utils import RawDog
 from pytgpt.imager import Imager
+from pytgpt.imager import Prodia
+from pytgpt.utils import Audio
 from WebChatGPT.console import chat as webchatgpt
 from colorama import Fore
 from colorama import init as init_colorama
@@ -66,6 +68,8 @@ class this:
 
     context_settings = dict(auto_envvar_prefix="PYTGPT")
 
+    image_providers_map: dict[str, object] = {"default": Imager, "prodia": Prodia}
+
     """Console utils"""
 
     @staticmethod
@@ -569,6 +573,26 @@ class Main(cmd.Cmd):
                     act=awesome_prompt,
                 )
 
+            elif provider == "yepchat":
+                from pytgpt.yepchat import main as yepchat
+
+                self.bot = yepchat.YEPCHAT(
+                    is_conversation=disable_conversation,
+                    max_tokens=max_tokens,
+                    temperature=temperature,
+                    presence_penalty=top_p,
+                    frequency_penalty=top_k,
+                    top_p=top_p,
+                    model=getOr(model, yepchat.model),
+                    timeout=timeout,
+                    intro=intro,
+                    filepath=filepath,
+                    update_file=update_file,
+                    proxies=proxies,
+                    history_offset=history_offset,
+                    act=awesome_prompt,
+                )
+
             elif provider == "gpt4all":
                 assert auth, (
                     "Path to LLM (.gguf or .bin) file is required. "
@@ -721,6 +745,9 @@ class Main(cmd.Cmd):
         self.confirm_script = confirm_script
         self.interpreter = interpreter
         self.rawdog = rawdog
+        self.read_aloud = False
+        self.read_aloud_voice = "Brian"
+        self.path_to_last_response_audio = None
         self.__init_time = time.time()
         self.__start_time = time.time()
         self.__end_time = time.time()
@@ -1034,6 +1061,13 @@ class Main(cmd.Cmd):
             is_json=True,
         )
 
+    @busy_bar.run(help="While rereading aloud", index=3, immediate=True)
+    def do_reread(self, line):
+        """Reread aloud last ai response"""
+        if not self.path_to_last_response_audio:
+            raise Exception("Path to last response audio is null")
+        Audio.play(self.path_to_last_response_audio)
+
     @busy_bar.run()
     def do_exec(self, line):
         """Exec python code in last response with RawDog"""
@@ -1144,9 +1178,24 @@ class Main(cmd.Cmd):
                 logging.error(this.getExc(e))
                 if exit_on_error:
                     sys.exit(1)
+
+            else:
+                self.post_default()
+
             finally:
                 self.__end_time = time.time()
 
+    @busy_bar.run(help="While reading aloud", immediate=True, index=3)
+    def post_default(self):
+        """Actions to be taken after upon successfull complete response generation triggered by `default` function"""
+        last_text: str = self.bot.get_message(self.bot.last_response)
+        if self.read_aloud and last_text is not None:
+            # Talk back to user
+            self.path_to_last_response_audio = Audio.text_to_audio(
+                last_text, voice=self.read_aloud_voice, auto=True
+            )
+            Audio.play(self.path_to_last_response_audio)
+
     def do_sys(self, line):
         """Execute system commands
         shortcut [./<command>]
@@ -1403,6 +1452,20 @@ class ChatInteractive:
         default="python",
         help="RawDog : Python's interpreter name",
     )
+    @click.option(
+        "-ttm",
+        "--talk-to-me",
+        is_flag=True,
+        help="Audiolize responses upon complete generation",
+    )
+    @click.option(
+        "-ttmv",
+        "--talk-to-me-voice",
+        help="The voice to use for speech synthesis",
+        type=click.Choice(Audio.all_voices),
+        metavar="|".join(Audio.all_voices[:8]),
+        default="Brian",
+    )
     @click.help_option("-h", "--help")
     def interactive(
         model,
@@ -1437,6 +1500,8 @@ class ChatInteractive:
         internal_exec,
         confirm_script,
         interpreter,
+        talk_to_me,
+        talk_to_me_voice,
     ):
         """Chat with AI interactively (Default)"""
         this.clear_history_file(filepath, new)
@@ -1471,6 +1536,8 @@ class ChatInteractive:
         bot.prettify = prettify
         bot.vertical_overflow = vertical_overflow
         bot.disable_stream = whole
+        bot.read_aloud = talk_to_me
+        bot.read_aloud_voice = talk_to_me_voice
         if prompt:
             if with_copied:
                 prompt = prompt + "\n" + clipman.get()
@@ -1685,6 +1752,20 @@ class ChatGenerate:
         default="python",
         help="RawDog : Python's interpreter name",
     )
+    @click.option(
+        "-ttm",
+        "--talk-to-me",
+        is_flag=True,
+        help="Audiolize responses upon complete generation",
+    )
+    @click.option(
+        "-ttmv",
+        "--talk-to-me-voice",
+        help="The voice to use for speech synthesis",
+        type=click.Choice(Audio.all_voices),
+        metavar="|".join(Audio.all_voices[:8]),
+        default="Brian",
+    )
     @click.help_option("-h", "--help")
     def generate(
         model,
@@ -1719,6 +1800,8 @@ class ChatGenerate:
         internal_exec,
         confirm_script,
         interpreter,
+        talk_to_me,
+        talk_to_me_voice,
     ):
         """Generate a quick response with AI"""
         this.clear_history_file(filepath, new)
@@ -1796,6 +1879,8 @@ class ChatGenerate:
         bot.prettify = prettify
         bot.vertical_overflow = vertical_overflow
         bot.disable_stream = whole
+        bot.read_aloud = talk_to_me
+        bot.read_aloud_voice = talk_to_me_voice
         bot.default(prompt, True, normal_stdout=(sys.stdout.isatty() == False))
 
 
@@ -2272,6 +2357,15 @@ class ImageGen:
     @staticmethod
     @click.command(context_settings=this.context_settings)
     @click.argument("prompt")
+    @click.option(
+        "-p",
+        "--provider",
+        help="Provider name for image generation",
+        type=click.Choice(
+            list(this.image_providers_map.keys()),
+        ),
+        default="default",
+    )
     @click.option(
         "-d",
         "--directory",
@@ -2293,7 +2387,7 @@ class ImageGen:
         type=click.IntRange(5, 300),
         help="Http request timeout in seconds",
     )
-    @click.option("-p", "--proxy", help="Http request proxy")
+    @click.option("-pr", "--proxy", help="Http request proxy")
     @click.option(
         "-nd",
         "--no-additives",
@@ -2303,16 +2397,17 @@ class ImageGen:
     @click.option("-q", "--quiet", is_flag=True, help="Suppress progress bar")
     @click.help_option("-h", "--help")
     def generate_image(
-        prompt, directory, amount, name, timeout, proxy, no_additives, quiet
+        prompt, provider, directory, amount, name, timeout, proxy, no_additives, quiet
     ):
         """Generate images with pollinations.ai"""
+        provider_obj = this.image_providers_map.get(provider)
         with Progress() as progress:
             task = progress.add_task(
                 f"[cyan]Generating ...[{amount}]",
                 total=amount,
                 visible=quiet == False,
             )
-            imager = Imager(timeout=timeout, proxies=proxy if proxy else {})
+            imager = provider_obj(timeout=timeout, proxies=proxy if proxy else {})
             for image in imager.generate(
                 prompt=prompt,
                 amount=amount,
@@ -2423,18 +2518,25 @@ class API:
 
     @staticmethod
     @click.command(context_settings=this.context_settings)
-    @click.argument("content", required=True, type=click.Choice(["images", "all"]))
+    @click.argument(
+        "content", required=True, type=click.Choice(["images", "audios", "all"])
+    )
     @click.option("-y", "--yes", is_flag=True, help="Okay to all confirmation prompts")
     @click.help_option("-h", "--help")
     def clear(content: str, yes: bool):
         """Delete api's static contents"""
-        from pytgpt.utils import api_static_image_dir, api_static_dir
+        from pytgpt.utils import (
+            api_static_image_dir,
+            api_static_audio_dir,
+            api_static_dir,
+        )
         from pathlib import Path
         import shutil
 
         static_contents_map: dict[str, Path] = {
             "images": api_static_image_dir,
             "all": api_static_dir,
+            "audios": api_static_audio_dir,
         }
         content_path: Path = static_contents_map[content]
         if not yes:
