@@ -11,7 +11,11 @@ import click
 from rich.markdown import Markdown
 from rich.console import Console
 from pathlib import Path
-import os
+from typing import Union
+from typing import NoReturn
+import requests
+import vlc
+from time import sleep as wait
 
 appdir = appdirs.AppDirs("pytgpt", "Smartwa")
 
@@ -21,9 +25,12 @@ api_static_dir = Path(default_path) / "api"
 
 api_static_image_dir = api_static_dir / "images"
 
+api_static_audio_dir = api_static_dir / "audios"
+
 os.makedirs(default_path, exist_ok=True)
 os.makedirs(api_static_dir.as_posix(), exist_ok=True)
 os.makedirs(api_static_image_dir.as_posix(), exist_ok=True)
+os.makedirs(api_static_audio_dir.as_posix(), exist_ok=True)
 
 
 def run_system_command(
@@ -178,9 +185,10 @@ class Conversation:
                 # Presume intro prompt is part of the file content
                 self.chat_history = file_contents
 
-    def __trim_chat_history(self, chat_history: str) -> str:
+    def __trim_chat_history(self, chat_history: str, intro: str = None) -> str:
         """Ensures the len(prompt) and max_tokens_to_sample is not > 4096"""
-        len_of_intro = len(self.intro)
+        intro = self.intro if intro is None else intro
+        len_of_intro = len(intro)
         len_of_chat_history = len(chat_history)
         total = (
             self.max_tokens_to_sample + len_of_intro + len_of_chat_history
@@ -189,24 +197,25 @@ class Conversation:
             truncate_at = (total - self.history_offset) + self.prompt_allowance
             # Remove head of total (n) of chat_history
             new_chat_history = chat_history[truncate_at:]
-            self.chat_history = self.intro + "\n... " + new_chat_history
+            self.chat_history = intro + "\n... " + new_chat_history
             # print(len(self.chat_history))
             return self.chat_history
         # print(len(chat_history))
         return chat_history
 
-    def gen_complete_prompt(self, prompt: str) -> str:
+    def gen_complete_prompt(self, prompt: str, intro: str = None) -> str:
         """Generates a kinda like incomplete conversation
 
         Args:
-            prompt (str): _description_
+            prompt (str): Chat prompt
+            intro (str): Override class' intro. Defaults to None.
 
         Returns:
             str: Updated incomplete chat_history
         """
         if self.status:
             resp = self.chat_history + self.history_format % dict(user=prompt, llm="")
-            return self.__trim_chat_history(resp)
+            return self.__trim_chat_history(resp, intro)
 
         return prompt
 
@@ -660,3 +669,348 @@ Current Datetime : {datetime.datetime.now()}
                         "error",
                     )
                     return f"PREVIOUS SCRIPT EXCEPTION:\n{str(e)}"
+
+
+class Audio:
+    # Request headers
+    cache_dir: Path = api_static_audio_dir
+    headers: dict[str, str] = {
+        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36"
+    }
+    all_voices: list[str] = [
+        "Filiz",
+        "Astrid",
+        "Tatyana",
+        "Maxim",
+        "Carmen",
+        "Ines",
+        "Cristiano",
+        "Vitoria",
+        "Ricardo",
+        "Maja",
+        "Jan",
+        "Jacek",
+        "Ewa",
+        "Ruben",
+        "Lotte",
+        "Liv",
+        "Seoyeon",
+        "Takumi",
+        "Mizuki",
+        "Giorgio",
+        "Carla",
+        "Bianca",
+        "Karl",
+        "Dora",
+        "Mathieu",
+        "Celine",
+        "Chantal",
+        "Penelope",
+        "Miguel",
+        "Mia",
+        "Enrique",
+        "Conchita",
+        "Geraint",
+        "Salli",
+        "Matthew",
+        "Kimberly",
+        "Kendra",
+        "Justin",
+        "Joey",
+        "Joanna",
+        "Ivy",
+        "Raveena",
+        "Aditi",
+        "Emma",
+        "Brian",
+        "Amy",
+        "Russell",
+        "Nicole",
+        "Vicki",
+        "Marlene",
+        "Hans",
+        "Naja",
+        "Mads",
+        "Gwyneth",
+        "Zhiyu",
+        "es-ES-Standard-A",
+        "it-IT-Standard-A",
+        "it-IT-Wavenet-A",
+        "ja-JP-Standard-A",
+        "ja-JP-Wavenet-A",
+        "ko-KR-Standard-A",
+        "ko-KR-Wavenet-A",
+        "pt-BR-Standard-A",
+        "tr-TR-Standard-A",
+        "sv-SE-Standard-A",
+        "nl-NL-Standard-A",
+        "nl-NL-Wavenet-A",
+        "en-US-Wavenet-A",
+        "en-US-Wavenet-B",
+        "en-US-Wavenet-C",
+        "en-US-Wavenet-D",
+        "en-US-Wavenet-E",
+        "en-US-Wavenet-F",
+        "en-GB-Standard-A",
+        "en-GB-Standard-B",
+        "en-GB-Standard-C",
+        "en-GB-Standard-D",
+        "en-GB-Wavenet-A",
+        "en-GB-Wavenet-B",
+        "en-GB-Wavenet-C",
+        "en-GB-Wavenet-D",
+        "en-US-Standard-B",
+        "en-US-Standard-C",
+        "en-US-Standard-D",
+        "en-US-Standard-E",
+        "de-DE-Standard-A",
+        "de-DE-Standard-B",
+        "de-DE-Wavenet-A",
+        "de-DE-Wavenet-B",
+        "de-DE-Wavenet-C",
+        "de-DE-Wavenet-D",
+        "en-AU-Standard-A",
+        "en-AU-Standard-B",
+        "en-AU-Wavenet-A",
+        "en-AU-Wavenet-B",
+        "en-AU-Wavenet-C",
+        "en-AU-Wavenet-D",
+        "en-AU-Standard-C",
+        "en-AU-Standard-D",
+        "fr-CA-Standard-A",
+        "fr-CA-Standard-B",
+        "fr-CA-Standard-C",
+        "fr-CA-Standard-D",
+        "fr-FR-Standard-C",
+        "fr-FR-Standard-D",
+        "fr-FR-Wavenet-A",
+        "fr-FR-Wavenet-B",
+        "fr-FR-Wavenet-C",
+        "fr-FR-Wavenet-D",
+        "da-DK-Wavenet-A",
+        "pl-PL-Wavenet-A",
+        "pl-PL-Wavenet-B",
+        "pl-PL-Wavenet-C",
+        "pl-PL-Wavenet-D",
+        "pt-PT-Wavenet-A",
+        "pt-PT-Wavenet-B",
+        "pt-PT-Wavenet-C",
+        "pt-PT-Wavenet-D",
+        "ru-RU-Wavenet-A",
+        "ru-RU-Wavenet-B",
+        "ru-RU-Wavenet-C",
+        "ru-RU-Wavenet-D",
+        "sk-SK-Wavenet-A",
+        "tr-TR-Wavenet-A",
+        "tr-TR-Wavenet-B",
+        "tr-TR-Wavenet-C",
+        "tr-TR-Wavenet-D",
+        "tr-TR-Wavenet-E",
+        "uk-UA-Wavenet-A",
+        "ar-XA-Wavenet-A",
+        "ar-XA-Wavenet-B",
+        "ar-XA-Wavenet-C",
+        "cs-CZ-Wavenet-A",
+        "nl-NL-Wavenet-B",
+        "nl-NL-Wavenet-C",
+        "nl-NL-Wavenet-D",
+        "nl-NL-Wavenet-E",
+        "en-IN-Wavenet-A",
+        "en-IN-Wavenet-B",
+        "en-IN-Wavenet-C",
+        "fil-PH-Wavenet-A",
+        "fi-FI-Wavenet-A",
+        "el-GR-Wavenet-A",
+        "hi-IN-Wavenet-A",
+        "hi-IN-Wavenet-B",
+        "hi-IN-Wavenet-C",
+        "hu-HU-Wavenet-A",
+        "id-ID-Wavenet-A",
+        "id-ID-Wavenet-B",
+        "id-ID-Wavenet-C",
+        "it-IT-Wavenet-B",
+        "it-IT-Wavenet-C",
+        "it-IT-Wavenet-D",
+        "ja-JP-Wavenet-B",
+        "ja-JP-Wavenet-C",
+        "ja-JP-Wavenet-D",
+        "cmn-CN-Wavenet-A",
+        "cmn-CN-Wavenet-B",
+        "cmn-CN-Wavenet-C",
+        "cmn-CN-Wavenet-D",
+        "nb-no-Wavenet-E",
+        "nb-no-Wavenet-A",
+        "nb-no-Wavenet-B",
+        "nb-no-Wavenet-C",
+        "nb-no-Wavenet-D",
+        "vi-VN-Wavenet-A",
+        "vi-VN-Wavenet-B",
+        "vi-VN-Wavenet-C",
+        "vi-VN-Wavenet-D",
+        "sr-rs-Standard-A",
+        "lv-lv-Standard-A",
+        "is-is-Standard-A",
+        "bg-bg-Standard-A",
+        "af-ZA-Standard-A",
+        "Tracy",
+        "Danny",
+        "Huihui",
+        "Yaoyao",
+        "Kangkang",
+        "HanHan",
+        "Zhiwei",
+        "Asaf",
+        "An",
+        "Stefanos",
+        "Filip",
+        "Ivan",
+        "Heidi",
+        "Herena",
+        "Kalpana",
+        "Hemant",
+        "Matej",
+        "Andika",
+        "Rizwan",
+        "Lado",
+        "Valluvar",
+        "Linda",
+        "Heather",
+        "Sean",
+        "Michael",
+        "Karsten",
+        "Guillaume",
+        "Pattara",
+        "Jakub",
+        "Szabolcs",
+        "Hoda",
+        "Naayf",
+    ]
+
+    @classmethod
+    def text_to_audio(
+        cls,
+        message: str,
+        voice: str = "Brian",
+        proxies: dict[str, str] = {},
+        timeout: int = 30,
+        save_to: Union[Path, str] = None,
+        auto: bool = False,
+    ) -> Union[str, bytes]:
+        """
+        Text to speech using StreamElements API
+
+        Parameters:
+            message (str): The text to convert to speech
+            voice (str, optional): The voice to use for speech synthesis. Defaults to "Brian".
+            proxies (dict, optional): Http request proxies. Default to {}.
+            timeout (int, optional): Http request timeout. Defaults to 30.
+            save_to (bool, optional): Path to save the audio file. Defaults to None.
+            auto (bool, optional): Generate filename for the contents based on `message` and save to `cls.cache_dir`. Defaults to False.
+
+        Returns:
+            result (Union[str, bytes]): Path to saved contents or audio content.
+        """
+        assert (
+            voice in cls.all_voices
+        ), f"Voice '{voice}' not one of [{', '.join(cls.all_voices)}]"
+        # Base URL for provider API
+        url: str = (
+            f"https://api.streamelements.com/kappa/v2/speech?voice={voice}&text={{{message}}}"
+        )
+        resp = requests.get(
+            url=url, headers=cls.headers, stream=True, proxies=proxies, timeout=timeout
+        )
+        if not resp.ok:
+            raise Exception(
+                f"Failed to perform the operation - ({resp.status_code}, {resp.reason}) - {resp.text}"
+            )
+
+        def sanitize_filename(path):
+            trash = [
+                "\\",
+                "/",
+                ":",
+                "*",
+                "?",
+                '"',
+                "<",
+                "|",
+                ">",
+            ]
+            for val in trash:
+                path = path.replace(val, "")
+            return path.strip()
+
+        if auto:
+            filename: str = message + "..." if len(message) <= 40 else message[:40]
+            save_to = cls.cache_dir / sanitize_filename(filename)
+            save_to = save_to.as_posix()
+
+        if save_to:
+            if not save_to.endswith("mp3"):
+                save_to += ".mp3"
+
+            with open(save_to, "wb") as fh:
+                for chunk in resp.iter_content(chunk_size=512):
+                    fh.write(chunk)
+        else:
+            return resp.content
+        return save_to
+
+    @staticmethod
+    def play(path_to_audio_file: Union[Path, str]) -> NoReturn:
+        """Play audio (.mp3) using vlc media player or termux-media-player
+
+        Args:
+            path_to_audio_file (Union[Path, str]): Path to audio file.
+
+        Returns:
+            NoReturn: None
+        """
+        if not Path(path_to_audio_file).is_file():
+            raise FileNotFoundError(f"File does not exist - '{path_to_audio_file}'")
+
+        def play_using_vlc():
+            # create instance
+            instance = vlc.Instance()
+            # Create a Media Player object
+            player = instance.media_player_new()
+            # Load the audio file
+            media = instance.media_new(path_to_audio_file)
+            # Set the media to the player
+            player.set_media(media)
+            # Play the audio file
+            player.play()
+            # Keep the script running until the audio finishes playing
+            while player.get_state() != vlc.State.Ended:
+                pass
+
+        def play_using_termux_media_player():
+            # check if media-player is installed
+            check = run_system_command(
+                "termux-media-player --help",
+                exit_on_error=False,
+                stdout_error=False,
+                help="Install termux-api package in order to play audio files.\n apt update && apt install termux-api",
+            )[1]
+
+            if check.returncode == 127:
+                raise Exception(
+                    "Install termux-api package in order to play audio files.\n apt update && apt install termux-api"
+                )
+            try:
+                run_system_command(f'termux-media-player play "{path_to_audio_file}"')
+                while (
+                    run_system_command("termux-media-player info")[1].stdout.strip()
+                    != "No track currently!"
+                ):
+                    wait(1)
+            except (KeyboardInterrupt, KeyError) as e:
+                logging.debug(f"Stopping termux-media-player due to - {e}")
+                run_system_command("termux-media-player stop")
+
+        (
+            play_using_termux_media_player()
+            if "termux" in os.environ.get("PREFIX", "")
+            else play_using_vlc()
+        )
